--------------------------------------------------------------------------------
-- Company:        Wakayama Univ. Robotics Lab.
-- Engineer:       M.Yamashita
-- Revision:       2007.08.18
-- Date:           2007.03.22
-- Design Name:    CMOS DSP Vision Sensor Board
-- Project Name:   WV9647
-- Target Device:  EPM7064AETC100-10
-- Tool versions:  Quartus II 5.1
-- Description:
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.ALL;
use ieee.std_logic_arith.ALL;
use ieee.std_logic_unsigned.ALL;

entity WV9647 is
  port(
    MCLK_in        : in std_logic;  -- 25MHz
    RESET_xin      : in std_logic;
    HSYNC_in       : in std_logic;
    VSYNC_in       : in std_logic;
    HSYNC_L_out,
    HSYNC_R_out    : out std_logic;
    VSYNC_L_out,
    VSYNC_R_out    : out std_logic;
    PCLK_L_in,
    PCLK_R_in      : in std_logic;
    CE2_xin,
    CE3_xin        : in std_logic;
    SSADS_xin      : in std_logic;
    SSWE_xin       : in std_logic;
    SSOE_xin       : in std_logic;
    DL_in,
    DR_in          : in std_logic_vector( 9 downto 0 );
    DU_out         : out std_logic_vector( 27 downto 18 );
    DL_out         : out std_logic_vector( 11 downto 02 );
    JP_xin         : in std_logic_vector( 1 to 4 );
    LED_xout       : out std_logic_vector( 2 to 4 )
  );
end WV9647;

architecture Behavior of WV9647 is

  ---------- system
  signal reset               : std_logic;
  signal clock               : std_logic;
  
  ---------- jumper select
  signal select_area         : std_logic;
  signal select_swap         : std_logic;
  signal select_transparent  : std_logic;
  signal select_fixed        : std_logic;

  ---------- image data
  signal pixel_u0,
         pixel_u1,
         pixel_u2,
         pixel_l0,
         pixel_l1,
         pixel_l2            : std_logic_vector( 9 downto 0 );
  constant PIXEL_INIT        : std_logic_vector( 9 downto 0 ) := "0000000000";
  constant PIXEL_HIZ         : std_logic_vector( 9 downto 0 ) := "ZZZZZZZZZZ";
  constant PIXEL_FIXED01     : std_logic_vector( 9 downto 0 ) := "0101010101";
  constant PIXEL_FIXED10     : std_logic_vector( 9 downto 0 ) := "1010101010";

  ---------- bus control
  signal chip_enable         : std_logic;
  signal output_enable       : std_logic;

begin

  -------------------------------------
  -- input
  -------------------------------------
  reset <= not RESET_xin;
  clock <= MCLK_in;
  select_area        <= not JP_xin(1);
  select_swap        <= not JP_xin(2);
  select_transparent <= not JP_xin(3);
  select_fixed       <= not JP_xin(4);

  pixel_u0 <= DL_in when select_swap = '0' else
              DR_in;
  pixel_l0 <= DR_in when select_swap = '0' else
              DL_in;

  -------------------------------------
  -- image data
  -------------------------------------
  process( reset, clock )
  begin
    if( reset = '1' ) then
      pixel_u1 <= PIXEL_INIT;
      pixel_l1 <= PIXEL_INIT;
    elsif( clock'event and clock = '1' ) then
      pixel_u1 <= pixel_u0;
      pixel_l1 <= pixel_l0;
    end if;
  end process;

  pixel_u2 <= pixel_u0 when( select_transparent = '1' ) else
              pixel_u1;
  pixel_l2 <= pixel_l0 when( select_transparent = '1' ) else
              pixel_l1;

  DU_out <= PIXEL_HIZ when( output_enable = '0' ) else
            pixel_u2  when( select_fixed = '0' )  else
            PIXEL_FIXED10;
  DL_out <= PIXEL_HIZ when( output_enable = '0' ) else
            pixel_l2  when( select_fixed = '0' )  else
            PIXEL_FIXED01;

  -------------------------------------
  -- chip select
  -- 1 clock delay
  -------------------------------------
  process( clock )
  begin
    if( clock'event and clock = '1' ) then
      if( select_area = '0' ) then
        chip_enable <= not CE2_xin;
      else
        chip_enable <= not CE3_xin;
      end if;
    end if;
  end process;

  output_enable <= chip_enable and ( not SSOE_xin );

  -------------------------------------
  -- output
  -------------------------------------
  HSYNC_L_out <= HSYNC_in;
  HSYNC_R_out <= HSYNC_in;
  VSYNC_L_out <= VSYNC_in;
  VSYNC_R_out <= VSYNC_in;

  LED_xout(2) <= not ( reset or HSYNC_in );
  LED_xout(3) <= not ( reset or VSYNC_in );
  LED_xout(4) <= not ( reset or output_enable );

end Behavior;
